use Mix.Config

# Configure your database
config :meta, Meta.Repo,
  username: "postgres",
  password: "postgres",
  database: "meta_dev",
  hostname: "localhost",
  show_sensitive_data_on_connection_error: true,
  pool_size: 10

# For development, we disable any cache and enable
# debugging and code reloading.
#
# The watchers configuration can be used to run external
# watchers to your application. For example, we use it
# with webpack to recompile .js and .css sources.
config :meta, MetaWeb.Endpoint,
  http: [port: 4000],
  debug_errors: true,
  code_reloader: true,
  check_origin: false,
  watchers: []

# ## SSL Support
#
# In order to use HTTPS in development, a self-signed
# certificate can be generated by running the following
# Mix task:
#
#     mix phx.gen.cert
#
# Note that this task requires Erlang/OTP 20 or later.
# Run `mix help phx.gen.cert` for more information.
#
# The `http:` config above can be replaced with:
#
#     https: [
#       port: 4001,
#       cipher_suite: :strong,
#       keyfile: "priv/cert/selfsigned_key.pem",
#       certfile: "priv/cert/selfsigned.pem"
#     ],
#
# If desired, both `http:` and `https:` keys can be
# configured to run both http and https servers on
# different ports.

# Watch static and templates for browser reloading.
config :meta, MetaWeb.Endpoint,
  live_reload: [
    patterns: [
      ~r"priv/static/.*(js|css|png|jpeg|jpg|gif|svg)$",
      ~r"priv/gettext/.*(po)$",
      ~r"lib/meta_web/(live|views)/.*(ex)$",
      ~r"lib/meta_web/templates/.*(eex)$"
    ]
  ]

# Do not include metadata nor timestamps in development logs
config :logger, :console, format: "[$level] $message\n"

# Set a higher stacktrace during development. Avoid configuring such
# in production as building large stacktraces may be expensive.
config :phoenix, :stacktrace_depth, 20

# Initialize plugs at runtime for faster development compilation
config :phoenix, :plug_init_mode, :runtime

# Send emails to local memory for development.
config :meta, Meta.Mailer,
  adapter: Bamboo.LocalAdapter

# LDAP configuration
config :meta, :ldap,
  server: "127.0.0.1",
  port: 3089,
  ssl: false,
  base_dn: "dc=example,dc=org",
  bind_dn: "cn=admin,dc=example,dc=org",
  bind_pw: "admin",
  default_group_gid: 1000

## Odoo API credentials
config :meta, :odoo,
  server: "https://odoo.staging.e-durable.ch",
  database: "e-Durable",
  user: "user",
  secret: "secret"
