defmodule Meta.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  def start(_type, _args) do
    children = [
      # Start the Ecto repository
      Meta.Repo,
      # Start the Telemetry supervisor
      MetaWeb.Telemetry,
      # Start the PubSub system
      {Phoenix.PubSub, name: Meta.PubSub},
      # Start the Endpoint (http/https)
      MetaWeb.Endpoint,
      # Starts LDAP connection pool
      Meta.LDAP,
      # Starts Odoo client
      Meta.Odoo
      # Start a worker by calling: Meta.Worker.start_link(arg)
      # {Meta.Worker, arg}
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: Meta.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  def config_change(changed, _new, removed) do
    MetaWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
