defmodule MetaWeb.Router do
  use MetaWeb, :router

  import MetaWeb.UserAuth

  pipeline :browser do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_flash
    plug :protect_from_forgery
    plug :put_secure_browser_headers
    plug :fetch_current_user
  end

  pipeline :browser_with_forgery do
    plug :accepts, ["html"]
    plug :fetch_session
    plug :fetch_flash
    plug :put_secure_browser_headers
    plug :fetch_current_user
  end

  pipeline :api do
    plug :accepts, ["json"]
  end

  pipeline :plain do
    plug :accepts, ["text"]
  end

  scope "/", MetaWeb do
    pipe_through :browser

    get "/", PageController, :index
    get "/support/new", SupportController, :new
  end

  scope "/", MetaWeb do
    pipe_through :browser_with_forgery

    # The static recycled.cloud website POST this endpoint, which basically is
    # Cross-Site Request Forgery (hence browser_with_forgery pipeline).
    post "/support/new", SupportController, :create
  end

  scope "/", MetaWeb do
    pipe_through :plain

    get "/keys/:username", UserKeysController, :index
  end

  ## Authentication routes

  scope "/", MetaWeb do
    pipe_through [:browser, :redirect_if_user_is_authenticated]

    get "/users/register", UserRegistrationController, :new
    post "/users/register", UserRegistrationController, :create
    get "/users/log_in", UserSessionController, :new
    post "/users/log_in", UserSessionController, :create
    get "/users/reset_password", UserResetPasswordController, :new
    post "/users/reset_password", UserResetPasswordController, :create
    get "/users/reset_password/:token", UserResetPasswordController, :edit
    put "/users/reset_password/:token", UserResetPasswordController, :update
  end

  scope "/", MetaWeb do
    pipe_through [:browser, :require_authenticated_user]

    get "/users/settings", UserSettingsController, :edit
    put "/users/settings", UserSettingsController, :update
    get "/users/settings/confirm_email/:token", UserSettingsController, :confirm_email
    post "/users/settings/keys/new", UserKeysController, :new
    get  "/users/settings/keys/:key_id/delete", UserKeysController, :delete
    get "/billing", BillingController, :index
    post "/billing/partner/update", BillingController, :update

    get "/hosting/vm", VirtualMachineController, :index
    get "/hosting/vm/:location/:id", VirtualMachineController, :show
  end

  scope "/", MetaWeb do
    pipe_through [:browser]

    get "/users/log_out", UserSessionController, :delete
    delete "/users/log_out", UserSessionController, :delete
    get "/users/confirm", UserConfirmationController, :new
    post "/users/confirm", UserConfirmationController, :create
    get "/users/confirm/:token", UserConfirmationController, :confirm
  end

  # In-memory mailer for development purposes only.
  # See https://hexdocs.pm/bamboo/Bamboo.SentEmailViewerPlug.html for details.
  if Meta.env() == :dev do
    forward "/outbox", Bamboo.SentEmailViewerPlug
  end
end
